@@ -7,11 +7,13 @@
 #' @param ... Variables used to find distinct rows.
 #' @param .keep_all If `TRUE` then all columns of data frame are kept,
 #' default is `FALSE`.
-#' @param sort Should result be sorted? Default is `FALSE`.
+#' @param .sort Should result be sorted? Default is `FALSE`.
 #' When `order = FALSE` this option has no effect on the result.
-#' @param order Should the groups be calculated as ordered groups?
+#' @param .order Should the groups be calculated as ordered groups?
 #' Setting to `TRUE` may sometimes offer a speed benefit, but usually this
 #' is not the case. The default is `FALSE`.
+#' @param order `r lifecycle::badge("superseded")` Use `.order`.
+#' @param sort `r lifecycle::badge("superseded")` Use `.sort`.
 #' @param .by (Optional). A selection of columns to group by for this operation.
 #' Columns are specified using tidy-select.
 #' @param .cols (Optional) alternative to `...` that accepts
@@ -23,8 +25,23 @@
 #'
 #' @export
 f_distinct <- function(data, ..., .keep_all = FALSE,
-                        sort = FALSE, order = sort,
-                        .by = NULL, .cols = NULL){
+                       .sort = FALSE, .order = sort,
+                       sort = .sort, order = .order,
+                       .by = NULL, .cols = NULL){
+  if (!identical(r_address(order), r_address(.order))){
+    lifecycle::deprecate_warn(
+      "0.3.0", what = "f_distinct(order)",
+      with = "f_distinct(.order)"
+    )
+    .order <- order
+  }
+  if (!identical(r_address(sort), r_address(.sort))){
+    lifecycle::deprecate_warn(
+      "0.3.0", what = "f_distinct(sort)",
+      with = "f_distinct(.sort)"
+    )
+    .sort <- sort
+  }
   n_dots <- dots_length(...)
   group_info <- tidy_group_info(data, ..., .by = {{ .by }},
                                 .cols = .cols,
@@ -49,7 +66,7 @@ f_distinct <- function(data, ..., .keep_all = FALSE,
 
   # If distinct variables are the same as group variables..
 
-  if (no_new_groups &&  sort == order && order == df_group_by_order_default(data)){
+  if (no_new_groups &&  .sort == .order && .order == df_group_by_order_default(data)){
     group_data <- group_data(data)
     if (.keep_all){
       unique_locs <- GRP_loc_starts(group_data(data)[[".rows"]])
@@ -63,11 +80,11 @@ f_distinct <- function(data, ..., .keep_all = FALSE,
     out_to_dedup <- f_select(out, .cols = dup_vars)
     # Using sort algorithm but returning order-of-first appearance groups
 
-    if (order && !sort){
+    if (.order && !.sort){
       unique_locs <- cheapr::which_val(row_id(out_to_dedup), 1L)
       slice <- !(length(unique_locs) == df_nrow(out) && is_sorted(unique_locs))
     } else {
-      if (order && sort){
+      if (.order && .sort){
         o <- radixorderv2(out_to_dedup, starts = TRUE)
         unique_locs <- o[attr(o, "starts")]
         slice <- !(length(unique_locs) == df_nrow(out) &&
